@@ -11,7 +11,7 @@ import qualified Prelude
 import           Helper
 import           Test.HUnit
 
-import           System.Directory (canonicalizePath, createDirectory)
+import           System.Directory (canonicalizePath)
 import           Data.Maybe
 import           Data.List
 import           Data.String.Interpolate
@@ -35,10 +35,15 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
       _foo:
         bar: 23
       library: {}
-      |] `shouldRenderTo` library [i|
-      other-modules:
-          Paths_foo
+      |] `shouldRenderTo` library_ [i|
       |]
+    it "warns on duplicate fields" $ do
+      [i|
+      name: foo
+      name: foo
+      |] `shouldWarn` [
+          "package.yaml: Duplicate field $.name"
+        ]
 
     describe "tested-with" $ do
       it "accepts a string" $ do
@@ -62,14 +67,6 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
           , GHC == 7.4.2
         |]
 
-    it "warns on duplicate fields" $ do
-      [i|
-      name: foo
-      name: foo
-      |] `shouldWarn` [
-          "package.yaml: Duplicate field $.name"
-        ]
-
     describe "handling of Paths_ module" $ do
       it "adds Paths_ to other-modules" $ do
         [i|
@@ -77,6 +74,7 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
         |] `shouldRenderTo` library [i|
         other-modules:
             Paths_foo
+        default-language: Haskell2010
         |]
 
       context "when cabal-version is >= 2" $ do
@@ -90,6 +88,7 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
               Paths_foo
           autogen-modules:
               Paths_foo
+          default-language: Haskell2010
           |]) { packageCabalVersion = "2.0" }
 
         context "when Paths_ module is listed explicitly under generated-other-modules" $ do
@@ -104,6 +103,7 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
                 Paths_foo
             autogen-modules:
                 Paths_foo
+            default-language: Haskell2010
             |]) { packageCabalVersion = "2.0" }
 
         context "when Paths_ module is listed explicitly under generated-exposed-modules" $ do
@@ -118,6 +118,7 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
                 Paths_foo
             autogen-modules:
                 Paths_foo
+            default-language: Haskell2010
             |]) { packageCabalVersion = "2.0" }
 
       context "when Paths_ is mentioned in a conditional that is always false" $ do
@@ -128,9 +129,10 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
             - condition: false
               other-modules: Paths_foo
           |] `shouldRenderTo` library [i|
+          default-language: Haskell2010
           |]
 
-      context "with RebindableSyntax and OverloadedStrings or OverloadedStrings" $ do
+      context "when Paths_ is used with RebindableSyntax and (OverloadedStrings or OverloadedLists)" $ do
         it "infers cabal-version 2.2" $ do
           [i|
           default-extensions: [RebindableSyntax, OverloadedStrings]
@@ -143,6 +145,7 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
               Paths_foo
           autogen-modules:
               Paths_foo
+          default-language: Haskell2010
           |]) {packageCabalVersion = "2.2"}
 
         context "when Paths_ is mentioned in a conditional that is always false" $ do
@@ -157,6 +160,7 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
             default-extensions:
                 RebindableSyntax
                 OverloadedStrings
+            default-language: Haskell2010
             |])
 
     describe "spec-version" $ do
@@ -342,11 +346,34 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
             github: sol/hpack-template
             path: defaults.yaml
             ref: "2017"
-        |] `shouldRenderTo` library [i|
+        |] `shouldRenderTo` library_ [i|
         exposed-modules:
             Foo
-        other-modules:
-            Paths_foo
+        |]
+
+      it "accepts executable defaults" $ do
+        writeFile "defaults/sol/hpack-template/2017/.hpack/defaults.yaml" [i|
+        main: Foo.hs
+        |]
+
+        [i|
+        executable:
+          defaults: sol/hpack-template@2017
+        |] `shouldRenderTo` executable_ "foo" [i|
+        main-is: Foo.hs
+        |]
+
+      it "gives `main` from executable section precedence" $ do
+        writeFile "defaults/sol/hpack-template/2017/.hpack/defaults.yaml" [i|
+        main: Foo.hs
+        |]
+
+        [i|
+        executable:
+          main: Bar.hs
+          defaults: sol/hpack-template@2017
+        |] `shouldRenderTo` executable_ "foo" [i|
+        main-is: Bar.hs
         |]
 
       it "accepts a list of defaults" $ do
@@ -439,9 +466,7 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
         defaults:
           local: defaults/foo.yaml
         library: {}
-        |] `shouldRenderTo` library [i|
-        other-modules:
-            Paths_foo
+        |] `shouldRenderTo` library_ [i|
         default-extensions:
             RecordWildCards
             DeriveFunctor
@@ -629,6 +654,19 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
           packageCabalVersion = "1.12"
         }
 
+      it "accepts build-tool-depends as an alias" $ do
+        [i|
+        executable:
+          build-tool-depends:
+            hspec-discover: 0.1.0
+        |] `shouldRenderTo` (executable_ "foo" [i|
+        build-tool-depends:
+            hspec-discover:hspec-discover ==0.1.0
+        |]) {
+          packageCabalVersion = "1.12"
+        , packageWarnings = ["package.yaml: $.executable.build-tool-depends is deprecated, use $.executable.build-tools instead"]
+        }
+
       context "when the name of a build tool matches an executable from the same package" $ do
         it "adds it to build-tools" $ do
           [i|
@@ -734,6 +772,17 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
             base
         |]
 
+      it "accepts build-depends as an alias" $ do
+        [i|
+        executable:
+          build-depends: base
+        |] `shouldRenderTo` (executable_ "foo" [i|
+        build-depends:
+            base
+        |]) {
+          packageWarnings = ["package.yaml: $.executable.build-depends is deprecated, use $.executable.dependencies instead"]
+        }
+
       it "accepts dependencies with subcomponents" $ do
         [i|
         executable:
@@ -794,6 +843,18 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
           , weston
         |]
 
+      it "accepts pkgconfig-depends as an alias" $ do
+        [i|
+        pkgconfig-depends:
+          - QtWebKit
+          - weston
+        executable: {}
+        |] `shouldRenderTo` executable_ "foo" [i|
+        pkgconfig-depends:
+            QtWebKit
+          , weston
+        |]
+
     describe "include-dirs" $ do
       it "accepts include-dirs" $ do
         [i|
@@ -820,6 +881,15 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
             bar.h
         |]
 
+    describe "ghc-shared-options" $ do
+      it "accepts ghc-shared-options" $ do
+        [i|
+        ghc-shared-options: -Wall
+        executable: {}
+        |] `shouldRenderTo` executable_ "foo" [i|
+        ghc-shared-options: -Wall
+        |]
+
     describe "js-sources" $ before_ (touch "foo.js" >> touch "jsbits/bar.js") $ do
       it "accepts js-sources" $ do
         [i|
@@ -867,9 +937,12 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
                 when:
                   condition: True
                   cxx-options: -Wall
-          |] `shouldRenderTo` (executable_ "foo" [i|
+          |] `shouldRenderTo` (executable "foo" [i|
+          other-modules:
+              Paths_foo
           autogen-modules:
               Paths_foo
+          default-language: Haskell2010
           if true
             if true
               if true
@@ -891,6 +964,63 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
             cxxbits/bar.cc
         |]) {packageCabalVersion = "2.2"}
 
+    describe "language" $ do
+      it "accepts language" $ do
+        [i|
+        language: GHC2021
+        executable: {}
+        |] `shouldRenderTo` executable "foo" [i|
+          other-modules:
+              Paths_foo
+          default-language: GHC2021
+        |]
+
+      it "omits language if it is null" $ do
+        [i|
+        language: null
+        executable: {}
+        |] `shouldRenderTo` executable "foo" [i|
+          other-modules:
+              Paths_foo
+        |]
+
+      it "accepts default-language as an alias" $ do
+        [i|
+        default-language: GHC2021
+        executable: {}
+        |] `shouldRenderTo` (executable "foo" [i|
+          other-modules:
+              Paths_foo
+          default-language: GHC2021
+        |]) {
+          packageWarnings = ["package.yaml: $.default-language is deprecated, use $.language instead"]
+        }
+
+      it "gives section-level language precedence" $ do
+        [i|
+        language: Haskell2010
+        executable:
+          language: GHC2021
+        |] `shouldRenderTo` executable "foo" [i|
+          other-modules:
+              Paths_foo
+          default-language: GHC2021
+        |]
+
+      it "accepts language from defaults" $ do
+        writeFile "defaults/sol/hpack-template/2017/.hpack/defaults.yaml" [i|
+        language: GHC2021
+        |]
+
+        [i|
+        defaults: sol/hpack-template@2017
+        library: {}
+        |] `shouldRenderTo` library [i|
+        other-modules:
+            Paths_foo
+        default-language: GHC2021
+        |]
+
     describe "extra-lib-dirs" $ do
       it "accepts extra-lib-dirs" $ do
         [i|
@@ -1054,6 +1184,7 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
               Foo
           other-modules:
               Foo
+          default-language: Haskell2010
           |]
 
       context "with mixins" $ do
@@ -1073,6 +1204,7 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
               foo
           mixins:
               foo (Blah as Etc)
+          default-language: Haskell2010
           |]) {packageCabalVersion = "2.0"}
 
     describe "internal-libraries" $ do
@@ -1137,6 +1269,7 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
               Foo
           other-modules:
               Paths_foo
+          default-language: Haskell2010
           |]
 
         it "ignores duplicate modules" $ do
@@ -1152,6 +1285,7 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
               Foo
           other-modules:
               Paths_foo
+          default-language: Haskell2010
           |]
 
         context "with exposed-modules" $ do
@@ -1170,6 +1304,7 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
             other-modules:
                 Bar
                 Paths_foo
+            default-language: Haskell2010
             |]
 
         context "with other-modules" $ do
@@ -1187,6 +1322,7 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
                 Foo
             other-modules:
                 Bar
+            default-language: Haskell2010
             |]
 
         context "with both exposed-modules and other-modules" $ do
@@ -1205,6 +1341,7 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
                 Foo
             other-modules:
                 Bar
+            default-language: Haskell2010
             |]
 
         context "with neither exposed-modules nor other-modules" $ do
@@ -1222,6 +1359,7 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
                 Foo
             other-modules:
                 Paths_foo
+            default-language: Haskell2010
             |]
 
         context "with a conditional" $ do
@@ -1236,15 +1374,17 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
                 exposed-modules:
                   - Foo
                   - Paths_foo
-            |] `shouldRenderTo` library [i|
-            hs-source-dirs:
-                src
-            if os(windows)
+            |] `shouldRenderTo` package [i|
+            library
+              hs-source-dirs:
+                  src
               exposed-modules:
-                  Foo
-                  Paths_foo
-            exposed-modules:
-                Bar
+                  Bar
+              default-language: Haskell2010
+              if os(windows)
+                exposed-modules:
+                    Foo
+                    Paths_foo
             |]
 
           context "with a source-dir inside the conditional" $ do
@@ -1255,14 +1395,16 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
                 when:
                   condition: os(windows)
                   source-dirs: windows
-              |] `shouldRenderTo` library [i|
-              other-modules:
-                  Paths_foo
-              if os(windows)
+              |] `shouldRenderTo` package [i|
+              library
                 other-modules:
-                    Foo
-                hs-source-dirs:
-                    windows
+                    Paths_foo
+                default-language: Haskell2010
+                if os(windows)
+                  other-modules:
+                      Foo
+                  hs-source-dirs:
+                      windows
               |]
 
             it "does not infer outer modules" $ do
@@ -1278,17 +1420,19 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
                   else:
                     source-dirs: unix/
 
-              |] `shouldRenderTo` library [i|
-              exposed-modules:
-                  Foo
-              other-modules:
-                  Paths_foo
-              if os(windows)
-                hs-source-dirs:
-                    windows/
-              else
-                hs-source-dirs:
-                    unix/
+              |] `shouldRenderTo` package [i|
+              library
+                exposed-modules:
+                    Foo
+                other-modules:
+                    Paths_foo
+                default-language: Haskell2010
+                if os(windows)
+                  hs-source-dirs:
+                      windows/
+                else
+                  hs-source-dirs:
+                      unix/
               |]
 
         context "with generated modules" $ do
@@ -1307,6 +1451,7 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
                 Paths_foo
                 Foo
                 Bar
+            default-language: Haskell2010
             |]) {packageCabalVersion = "2.0"}
 
           it "does not infer any mentioned generated modules" $ do
@@ -1329,6 +1474,7 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
                 Paths_foo
                 Exposed
                 Other
+            default-language: Haskell2010
             |]) {packageCabalVersion = "2.0"}
 
           it "does not infer any generated modules mentioned inside conditionals" $ do
@@ -1341,21 +1487,23 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
                 condition: os(windows)
                 generated-exposed-modules: Exposed
                 generated-other-modules: Other
-            |] `shouldRenderTo` (library [i|
-            other-modules:
-                Paths_foo
-            autogen-modules:
-                Paths_foo
-            hs-source-dirs:
-                src
-            if os(windows)
-              exposed-modules:
-                  Exposed
+            |] `shouldRenderTo` (package [i|
+            library
               other-modules:
-                  Other
+                  Paths_foo
               autogen-modules:
-                  Other
-                  Exposed
+                  Paths_foo
+              hs-source-dirs:
+                  src
+              default-language: Haskell2010
+              if os(windows)
+                exposed-modules:
+                    Exposed
+                other-modules:
+                    Other
+                autogen-modules:
+                    Other
+                    Exposed
             |]) {packageCabalVersion = "2.0"}
 
       context "with an executable" $ do
@@ -1374,6 +1522,7 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
             other-modules:
                 Foo
                 Paths_foo
+            default-language: Haskell2010
           |]
 
         it "allows to specify other-modules" $ do
@@ -1391,6 +1540,7 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
                 src
             other-modules:
                 Baz
+            default-language: Haskell2010
           |]
 
         it "does not infer any mentioned generated modules" $ do
@@ -1411,6 +1561,7 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
             autogen-modules:
                 Paths_foo
                 Foo
+            default-language: Haskell2010
           |]) {packageCabalVersion = "2.0"}
 
         context "with a conditional" $ do
@@ -1430,6 +1581,7 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
                 Paths_foo
             hs-source-dirs:
                 src
+            default-language: Haskell2010
             if os(windows)
               other-modules:
                   Foo
@@ -1451,6 +1603,7 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
                 Paths_foo
             hs-source-dirs:
                 src
+            default-language: Haskell2010
             if os(windows)
               other-modules:
                   Bar
@@ -1459,6 +1612,16 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
             |]
 
     describe "executables" $ do
+      it "accepts main-is as an alias for main" $ do
+        [i|
+        executable:
+          main-is: Foo.hs
+        |] `shouldRenderTo` (executable_ "foo" [i|
+        main-is: Foo.hs
+        |]) {
+          packageWarnings = ["package.yaml: $.executable.main-is is deprecated, use $.executable.main instead"]
+        }
+
       it "accepts arbitrary entry points as main" $ do
         touch "src/Foo.hs"
         touch "src/Bar.hs"
@@ -1475,6 +1638,7 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
         other-modules:
             Bar
             Paths_foo
+        default-language: Haskell2010
         |]
 
       context "with a conditional" $ do
@@ -1487,8 +1651,11 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
               when:
                 condition: os(windows)
                 main: Foo.hs
-          |] `shouldRenderTo` executable_ "foo" [i|
+          |] `shouldRenderTo` executable "foo" [i|
           ghc-options: -Wall
+          other-modules:
+              Paths_foo
+          default-language: Haskell2010
           if os(windows)
             main-is: Foo.hs
           |]
@@ -1500,7 +1667,10 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
               when:
                 condition: os(windows)
                 main: Foo
-          |] `shouldRenderTo` executable_ "foo" [i|
+          |] `shouldRenderTo` executable "foo" [i|
+          other-modules:
+              Paths_foo
+          default-language: Haskell2010
           if os(windows)
             main-is: Foo.hs
             ghc-options: -main-is Foo
@@ -1513,7 +1683,10 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
           condition: os(windows)
           dependencies: Win32
         executable: {}
-        |] `shouldRenderTo` executable_ "foo" [i|
+        |] `shouldRenderTo` executable "foo" [i|
+        other-modules:
+            Paths_foo
+        default-language: Haskell2010
         if os(windows)
           build-depends:
               Win32
@@ -1547,7 +1720,10 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
             else:
               dependencies: unix
           executable: {}
-          |] `shouldRenderTo` executable_ "foo" [i|
+          |] `shouldRenderTo` executable "foo" [i|
+          other-modules:
+              Paths_foo
+          default-language: Haskell2010
           if os(windows)
             build-depends:
                 Win32
@@ -1768,11 +1944,8 @@ instance Show RenderResult where
 shouldRenderTo :: HasCallStack => String -> Package -> Expectation
 shouldRenderTo input p = do
   writeFile packageConfig ("name: foo\n" ++ unindent input)
-  let currentDirectory = ".working-directory"
-  createDirectory currentDirectory
-  withCurrentDirectory currentDirectory $ do
-    (warnings, output) <- run ".." (".." </> packageConfig) expected
-    RenderResult warnings (dropEmptyLines output) `shouldBe` RenderResult (packageWarnings p) expected
+  (warnings, output) <- run "" packageConfig expected
+  RenderResult warnings (dropEmptyLines output) `shouldBe` RenderResult (packageWarnings p) expected
   where
     expected = dropEmptyLines (renderPackage p)
     dropEmptyLines = unlines . filter (not . null) . lines
@@ -1813,7 +1986,6 @@ library l = package content
     content = [i|
 library
 #{indentBy 2 $ unindent l}
-  default-language: Haskell2010
 |]
 
 internalLibrary :: String -> String -> Package
@@ -1842,7 +2014,6 @@ executable name e = package content
     content = [i|
 executable #{name}
 #{indentBy 2 $ unindent e}
-  default-language: Haskell2010
 |]
 
 package :: String -> Package
@@ -1872,7 +2043,7 @@ indentBy n = unlines . map (replicate n ' ' ++) . lines
 
 license :: String
 license = [i|
-Copyright (c) 2014-2018 Simon Hengel <sol@typeful.net>
+Copyright (c) 2014-2023 Simon Hengel <sol@typeful.net>
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
