@@ -1,7 +1,9 @@
 {-# LANGUAGE FlexibleContexts #-}
 {-# LANGUAGE ConstraintKinds #-}
+{-# OPTIONS_GHC -fno-warn-orphans #-}
 module Helper (
-  module Test.Hspec
+  module Imports
+, module Test.Hspec
 , module Test.Mockery.Directory
 , module Control.Monad
 , module Control.Applicative
@@ -11,6 +13,8 @@ module Helper (
 , yaml
 ) where
 
+import           Imports
+
 import           Test.Hspec
 import           Test.Mockery.Directory
 import           Control.Monad
@@ -23,6 +27,11 @@ import           System.FilePath
 import           Data.Yaml.TH (yamlQQ)
 import           Language.Haskell.TH.Quote (QuasiQuoter)
 
+import           Hpack.Config
+
+instance IsString Cond where
+  fromString = CondExpression
+
 withCurrentDirectory :: FilePath -> IO a -> IO a
 withCurrentDirectory dir action = do
   bracket getCurrentDirectory setCurrentDirectory $ \ _ -> do
